import data, input, graphics, rope, game, menu, sound
import pygame
from pygame.locals import * 
from constants import *

class controller:
    def __init__(self):
        pygame.init()
        self.clock = pygame.time.Clock()

        screenWidth = 800
        screenHeight = 600
    
        self.graphicsThing = graphics.graphicsClass(screenWidth,screenHeight)
        self.inputThing = input.inputClass()
        self.soundThing = sound.sound()
        self.soundThing.playRandom()
        self.fontThing = graphics.fontClass()
        self.gameThing = None

        self.mainMenu = menu.menu()
        self.gameMenu = menu.menu(self.mainMenu)
        self.musicMenu = menu.menu(self.mainMenu)
        #nothing = menu.normalMenuItem("Nothing",self,"theGoggles")

        quit = menu.normalMenuItem("Quit",self,"quitGame")
        newgame = menu.subMenuItem("New",self.gameMenu)
        music = menu.subMenuItem("Music",self.musicMenu)

        normal = menu.normalMenuItem("Normal",self,"newGame")
        e1 = menu.normalMenuItem("Endurance (easy)",self,"endurance1")
        e2 = menu.normalMenuItem("Endurance (hard)",self,"endurance2")
        e3 = menu.normalMenuItem("Endurance (nearly impossible)",self,"endurance3")

        track1 = menu.normalMenuItem("Track 1",self,"playTrack1")
        track2 = menu.normalMenuItem("Track 2",self,"playTrack2")
        track3 = menu.normalMenuItem("Track 3",self,"playTrack3")
        track4 = menu.normalMenuItem("Track 4",self,"playTrack4")
        track5 = menu.normalMenuItem("Track 5",self,"playTrack5")

        self.gameMenu.addItem(normal)
        self.gameMenu.addItem(e1)
        self.gameMenu.addItem(e2)
        self.gameMenu.addItem(e3)

        self.musicMenu.addItem(track1)
        self.musicMenu.addItem(track2)
        self.musicMenu.addItem(track3)
        self.musicMenu.addItem(track4)
        self.musicMenu.addItem(track5)

        self.mainMenu.addItem(newgame)
        #self.mainMenu.addItem(nothing)
        self.mainMenu.addItem(music)
        self.mainMenu.addItem(quit)

        self.paused = True
        self.done = False

    def playTrack1(self):
        self.soundThing.playTrack(0)

    def playTrack2(self):
        self.soundThing.playTrack(1)

    def playTrack3(self):
        self.soundThing.playTrack(2)

    def playTrack4(self):
        self.soundThing.playTrack(3)

    def playTrack5(self):
        self.soundThing.playTrack(4)

    def newGame(self):
        """Start a new game"""
        self.gameThing = game.gameClass(self.inputThing,self.graphicsThing,self.fontThing,self.soundThing)
        self.gameThing.pause(0) #treat it as if its been paused since forever
        self.paused = False

    def endurance1(self):
        """Start a new game"""
        self.gameThing = game.gameClass(self.inputThing,self.graphicsThing,self.fontThing,self.soundThing,mode="endurance1")
        self.gameThing.pause(0) #treat it as if its been paused since forever
        self.paused = False

    def endurance2(self):
        """Start a new game"""
        self.gameThing = game.gameClass(self.inputThing,self.graphicsThing,self.fontThing,self.soundThing,mode="endurance2")
        self.gameThing.pause(0) #treat it as if its been paused since forever
        self.paused = False

    def endurance3(self):
        """Start a new game"""
        self.gameThing = game.gameClass(self.inputThing,self.graphicsThing,self.fontThing,self.soundThing,mode="endurance3")
        self.gameThing.pause(0) #treat it as if its been paused since forever
        self.paused = False

    def quitGame(self):
        """Exit the game"""
        self.done = True

    def theGoggles(self):
        """THE GOGGLES! THEY DO NOTHING!"""
        pass

    def doStuff(self):
        """Show the menu or game"""
        if self.done:
            pygame.quit()
            return False #user has quit via the menu

        self.clock.tick()
        #print self.clock.get_fps()
        now = pygame.time.get_ticks() #time since pygame started
        dt = self.clock.get_time() #time between last two ticks
        
        self.inputThing.check() #check for input since last tick
        size = self.inputThing.newSize()
        if size:
            oldwidth = self.graphicsThing.screenWidth()
            oldheight = self.graphicsThing.screenHeight()
            xscale = float(size[0])/oldwidth #Damn you, integer division!
            yscale = float(size[1])/oldheight
            self.graphicsThing.resize(size)
            if self.gameThing:
                self.gameThing.resize(xscale,yscale)

        if self.gameThing and self.inputThing.keyPressed("escape"): #toggle menu if game has begun
            self.paused = not self.paused
            if self.paused:
                self.gameThing.pause(now)
                self.mainMenu.reset() #go back to the top level menu
        elif self.inputThing.done: #user has quit via the x at the top of the window
            self.done = True

        if self.paused or not self.gameThing: #show menu
            self.mainMenu.input(self.inputThing)
            self.mainMenu.draw(self.graphicsThing,self.fontThing)
        else: #play game
            self.gameThing.drawBG()
            self.gameThing.run(dt,now)
            self.gameThing.drawFG()

        self.graphicsThing.updateScreen()
        return True
