import pygame
import data, input, graphics, rope, random, math
from pygame.locals import * 
from constants import *

class level:
    """Stores information about the current level"""
    def __init__(self,dt,speed,number,spectrum=True):
        self.timeInterval = dt #time between new strings
        self.speed = speed #speed at which strings move
        self.number = number #number of strings per level
        self.spectrum = spectrum #is the spectrum visible?

class message:
    """Message that is shown on the screen for a while"""
    def __init__(self,text,font,fontsize,col,pos,expireTime):
        self.text = text
        self.font = font
        self.fontsize = fontsize
        self.pos = pos
        self.col = col
        self.expireTime = expireTime

class gameClass:
    """The main game"""
    def __init__(self,inputThing,graphicsThing,fontThing,soundThing,mode="normal"):
        self.inputThing = inputThing
        self.graphicsThing = graphicsThing
        self.fontThing = fontThing
        self.soundThing = soundThing
        self.paused = False
        self.pauseTime = 0
        self.mode = mode

        self.ducks = []
        self.nextDuckTime = random.gauss(duckAve,duckSigma)

        width = self.graphicsThing.screenWidth()
        height = self.graphicsThing.screenHeight()
        pygame.mouse.set_visible(False) #hide the cursor

        #level information
        self.levels = []
        if self.mode == "normal":
            self.levels.append(level(2000,0.075,10,True)) #time between strings, speed, number of strings)
            self.levels.append(level(2000,0.1,10 ,True))
            self.levels.append(level(2000,0.125,10,True)) 
            self.levels.append(level(1500,0.125,10,True))
            self.levels.append(level(1000,0.125,10,True))
            self.levels.append(level(1000,0.15,10,True))
            self.levels.append(level(900,0.15,10,True))
            self.levels.append(level(900,0.175,10,True))
            self.levels.append(level(800,0.175,10,True))
            self.levels.append(level(800,0.2,10,True))
            self.levels.append(level(700,0.2,10,True))
            self.levels.append(level(700,0.225,10,True))
            self.levels.append(level(600,0.225,10,True))
            self.levels.append(level(600,0.25,10,True))
            self.levels.append(level(500,0.25,10,True))
        elif self.mode == "endurance1":
            self.levels.append(level(900,0.15,10,True))
        elif self.mode == "endurance2":
            self.levels.append(level(600,0.225,10,True))
        elif self.mode == "endurance3":
            self.levels.append(level(600,0.225,10,False))
#        self.levels.append(level(500,0.275,10,True))
#        self.levels.append(level(500,0.3,10,True))

        #progress
        self.score = 0

        if self.mode == "normal":
            self.lives = 3
        else:
            self.lives = 0
        self.level = 0
        self.stringsCut = 0

        #messages which show you how well you did
        self.feedback = []

        self.bigMessage = None #message in the centre

        self.beamVisible = False #whether the laser is being fired
        self.fireTime = 0 #time the laser was last activated

        self.conveyorPos = 0

        #load fonts
        #self.normalFont = fontThing.makeFont("VeraMono.ttf",12)
        #self.messageFont = fontThing.makeFont("VeraIt.ttf",16)
        #self.bigFont = fontThing.makeFont("VeraBd.ttf",24)

        #load images
        file = data.filepath("laser.bmp")
        self.laserimg = graphicsThing.loadImage(file)
        file = data.filepath("conveyor1.bmp")
        self.conveyorimg = graphicsThing.loadImage(file)
        file = data.filepath("floor.bmp")
        self.floorimg = graphicsThing.loadImage(file)

        #draw spectrum image
        self.spectrum = graphicsThing.makeSpectrum(0, width, 0, 50)
        self.grey = self.graphicsThing.makeGrey(0, width, 0, 50)

        #make first string
        newString = rope.rope((0,0), height-50, self.levels[self.level].speed, width-1)
        self.strings=[newString]
        self.lastStringTime = 0

    def resize(self,xscale,yscale):
        width = self.graphicsThing.screenWidth()
        height = self.graphicsThing.screenHeight()

        #resize spectrum
        self.spectrum = self.graphicsThing.makeSpectrum(0, width, 0, 50)
        self.grey = self.graphicsThing.makeGrey(0, width, 0, 50)

        #resize strings
        for i in self.strings:
            i.length = width
            i.requiredLength = int(i.hue/360 * i.length)

        #move ducks
        for i in self.ducks:
            x = i.rect.centerx
            y = i.rect.centery
#            print x
#            print x*xscale
            i.rect.centerx = x*xscale
            i.rect.centery = y*yscale

    def pause(self,now):
        """Saves the current time so the game can carry on from where it left off next time run() is called"""
        self.paused = True
        self.pauseTime = now

    def judge(self,percentage,pos,now):
        """Show feedback messages and award points / reduce lives"""
        if percentage <= great:
            self.score += greatscore
            msg = "Precise!"
        elif percentage <= good:
            self.score += goodscore
            msg = "Good!"
        elif percentage <= acceptable:
            self.score += acceptablescore
            msg = "That'll do"
        else:
            self.lives -= 1
            if not self.gameOver():
                self.stringsCut -= 1 #when strings go off screen they always increment this, even if the string didnt count. instead of fixing that I'll just subtract 1 here OH GOD THIS CODE IS SO BAD
            msg = "Fail!"
        newMessage = message(msg,"VeraIt.ttf",16,(0,0,0),pos,now+messageDuration)
        self.feedback.append(newMessage)

    def bonus(self):
        """Change the speed to one from a randomly chosen level, and gain soem points"""
        self.score += duckscore
        randLevel = random.choice(self.levels)
        self.levels[self.level].speed = randLevel.speed

        #make the old strings move at the new speed
        if self.level < len(self.levels):
            for s in self.strings:
                s.speed = self.levels[self.level].speed


    def gameOver(self):
        if self.lives<0:
            return True
        else:
            return False

    def wonGame(self):
        if self.level >= len(self.levels):
            return True
        else:
            return False

    def drawBG(self):
        """Draw background"""
        width = self.graphicsThing.screenWidth()
        height = self.graphicsThing.screenHeight()

        self.graphicsThing.drawTiled(self.conveyorimg, (0,0), (conveyorWidth,height-50), (0,self.conveyorPos))
        self.graphicsThing.drawTiled(self.floorimg, (conveyorWidth,0), (width-conveyorWidth,height-50), (0,0))
        self.graphicsThing.drawTiled(self.conveyorimg, (width-conveyorWidth,0), (width,height-50), (0,self.conveyorPos))

    def drawFG(self):
        """Draw foreground"""
        width = self.graphicsThing.screenWidth()
        height = self.graphicsThing.screenHeight()

        if self.mode == "normal":
            #write score
            self.fontThing.writeText("Score:","VeraMono.ttf",12,(0,0,0),(width-50,20))
            self.fontThing.writeText(str(self.score),"VeraMono.ttf",12,(0,0,0),(width-20,20), True, False)

            lives = self.lives
            if self.gameOver():
                lives += 1 #can't have less than 0
            self.fontThing.writeText("Extras:","VeraMono.ttf",12,(0,0,0),(width-54,40))
            self.fontThing.writeText(str(lives),"VeraMono.ttf",12,(0,0,0),(width-20,40), True, False)

            level = self.level
            if not self.wonGame(): #level variable starts at 0
                level += 1
            self.fontThing.writeText("Level:","VeraMono.ttf",12,(0,0,0),(width-50,60))
            self.fontThing.writeText(str(level),"VeraMono.ttf",12,(0,0,0),(width-20,60), True, False)
        else:
            self.fontThing.writeText(str(self.stringsCut),"VeraBd.ttf",16,(255,255,255),(20,20), True, False)

    def finishedString(self,now):
        """Increase count and check to see if thats enough to move onto next level"""
        self.stringsCut += 1
        if self.stringsCut >= self.levels[self.level].number and self.mode == "normal": #go to next level
            self.level += 1
            self.lives = 3
            self.stringsCut = 0
            
            height = self.graphicsThing.screenHeight()
            width = self.graphicsThing.screenWidth()
            self.bigMessage=message("Level complete!","VeraBd.ttf",24,(0,0,0),(width/2,height/2),now+3*messageDuration)

            #increase the speed of the remaining strings to match the ones on the new level
            if self.level < len(self.levels):
                for s in self.strings:
                    s.speed = self.levels[self.level].speed

    def run(self,dt,now):
        """Continue the game"""            

        #correct for time lost while the game was paused
        if self.paused:
            self.paused = False
            lostTime = now - self.pauseTime
            self.lastStringTime += lostTime
            for m in self.feedback:
                m.expireTime += lostTime

        height = self.graphicsThing.screenHeight()
        width = self.graphicsThing.screenWidth()

        #move laser to current position
        laserpos = (self.inputThing.mousex(), height - 50)
        
        if self.inputThing.mousePressed() and now-self.fireTime > beamTime + rechargeTime: #fire the laser!!
            self.fireTime = pygame.time.get_ticks()
            self.beamVisible = True
            if not self.gameOver() and not self.wonGame():
                self.soundThing.laserSound()

        if not self.gameOver() and not self.wonGame(): #keep doing game stuff
            #laser beam stuff
            if self.beamVisible: 
                endpoint = 0
                hitString = None
                hitDuck = None

                #work out which string it hits, if any
                for i in self.strings:
                    if i.getPos() < height-100:
                        if not i.isCut():
                            endpoint = i.getPos() #laser is stopped by string
                            hitString = i
                            break
                        elif i.collide(laserpos[0]): #laser is stopped by cut string (cant cut again so do nothing (should make recharge time > removal time to cover this))
                            break

                if self.ducks:
                    for i in self.ducks:
                        y = i.collidePoint(laserpos[0])   
                        if y != None and y > endpoint and y < height-100:
                            hitDuck = i
                            endpoint = y
                            break

                #show the laser for a while before cutting the string
                if (now - self.fireTime) < beamTime:
                    self.graphicsThing.drawLine(lasercolor,(laserpos[0],laserpos[1]-51),(laserpos[0],endpoint), 3) #draw the laser beam
                else:
                    self.beamVisible = False
                    if hitDuck:
                        hitDuck.die(now)
                        self.soundThing.duckSound()
                        self.bonus()
                    elif hitString:
                        error = hitString.cutAt(laserpos[0]) #cut string
                        self.judge(error,(laserpos[0],endpoint), now)

            #Make new strings
            if now-self.lastStringTime > self.levels[self.level].timeInterval:
                newString = rope.rope((0,0), height-50, self.levels[self.level].speed, width-1)
                self.strings.append(newString)
                self.lastStringTime = now
        
            for i in self.strings[:]: #copy the list so that items can be removed while iterating over it
                if i.offscreen(): #has gone past the bottom edge
                    cut = i.isCut()
                    del self.strings[0] #since no string can overtake another i will always be the first string still in the list
                    if cut:
                        self.finishedString(now)
                    else:
                        self.lives -= 1
                elif i.isGone(): #has been cut and moved offscreen
                    j = self.strings.index(i)
                    del self.strings[j]
                    self.finishedString(now)
                else:
                    i.draw(self.graphicsThing)
                    i.move(dt)

        #big messages in the center
            if self.bigMessage and not self.gameOver() and not self.wonGame():
                if self.bigMessage.expireTime > now:
                    self.fontThing.writeText(self.bigMessage.text,self.bigMessage.font,self.bigMessage.fontsize,self.bigMessage.col,self.bigMessage.pos)
                else:
                    self.bigMessage = None

        #game has ended so just draw the old strings
        else:
            for i in self.strings:
                if i.offscreen() or i.isGone():
                    index = self.strings.index(i)
                    del self.strings[index]
                else:
                    i.draw(self.graphicsThing)
                    i.move(dt)

        #show feedback messages
        if self.feedback:
            for m in self.feedback[:]:
                if m.expireTime > now:
                    self.fontThing.writeText(m.text,m.font,m.fontsize,m.col,m.pos)
                else:
                    i = self.feedback.index(m)
                    del self.feedback[i]

        #big game over / won game messages that dont go away
        if self.gameOver():
            self.fontThing.writeText("Game over!","VeraBd.ttf",24,(0,0,0),(width/2,height/2))
        elif self.wonGame():
            self.fontThing.writeText("You win!","VeraBd.ttf",24,(0,0,0),(width/2,height/2))

        #move background image
        if self.wonGame():
            level = self.levels[-1]
        else:
            level = self.levels[self.level]
        self.conveyorPos += level.speed * dt
        
        #decide whether to add a new duck
        #l = (duckRate * dt)/1000 #average number of ducks which should have gone by
        #noDuckProb = math.exp(-l)
        #duckProb = 1-math.exp(-l)
        #r = random.random()
        if now > self.nextDuckTime and self.mode == "normal":
            x = random.randint(-130,-80)
            y = random.randint(0,height-121)
            vx = 0.1
            vy = self.levels[self.level].speed
            if random.choice([True,False]): #half the time it comes from the left, half right
                vx *= -1
                x = (width-x)-100
            self.ducks.append(graphics.ducky((x,y), 2, (vx,vy)))
            self.nextDuckTime = now + random.gauss(duckAve,duckSigma)

        for i in self.ducks:
            if not i.finished():
                i.update(now)
                i.draw()
            else:
                del self.ducks[self.ducks.index(i)]

        #draw spectrum & laser
        if self.levels[self.level].spectrum:
            self.graphicsThing.drawImage(self.spectrum, (0,height-50) ,center=False)        
        else:
            #gotta draw something else to hide the fact that the tiled images dont go all the way to the bottom... shhh!
            self.graphicsThing.drawImage(self.grey, (0,height-50) ,center=False)
        self.graphicsThing.drawImage(self.laserimg, laserpos) 

