import pygame
from color import hsv2rgb
from pygame.locals import *
from constants import *
import math
import data, os, string

class graphicsClass:
    """Handles graphics"""
    def __init__(self,width,height):
        self.surfaces = []
        self.width = width
        self.height = height
        self.window = pygame.display.set_mode((width, height), pygame.RESIZABLE) 
        self.screen = pygame.display.get_surface()
        pygame.display.set_caption('Final Cut') 
        #self.screen.fill(BGCOLOR)
        #pygame.display.flip()

    def screenHeight(self):
        return self.height

    def screenWidth(self):
        return self.width

    def resize(self,size):
        self.width = size[0]
        self.height = size[1]
        pygame.display.set_mode(size, pygame.RESIZABLE)
        self.screen = pygame.display.get_surface()

    def loadImage(self,filename):
        """loadImage(filename)
        Loads the image at filename into memory and returns an id number for it."""
        surface = pygame.image.load(filename)
        self.surfaces.append(surface)
        return len(self.surfaces) - 1

    def drawImage(self, surfaceid, pos, center=True):
        """drawImage(surfaceid, center)
        Draws the surface specified by surfaceid. pos is a tuple (x,y). If center is true the image will be centered at these coordinates."""
        surface = self.surfaces[surfaceid]
        size = surface.get_size()
        if center:
            pos = (pos[0]-size[0]/2,pos[1]-size[1]/2)
        self.screen.blit(surface, pos)

    def updateScreen(self):
        """updateScreen()
        Calls pygame.display.flip() and then clears the surface for next time"""
        pygame.display.flip()
        self.screen.fill(BGCOLOR)
    
    def drawLine(self, col, start, finish, width, surfaceid=-1):
        """drawLine(col, start, finish, width, [surfaceid])
        Draw a line onto the screen or another surface if specified."""
        if len(self.surfaces) > surfaceid >= 0:
            dest = self.surfaces[surfaceid]
        else:
            dest = self.screen
        pygame.draw.line(dest, col, start, finish, width)

    def makeSpectrum(self, start, finish, y, height):
        """drawSpectrum(start,finish,height,[surfaceid]
        Draw a spectrum onto the screen or another surface if specified."""
        id = -1
        width = finish-start
        if width > 0:
            #create new surface
            surface = pygame.Surface((width, height))
            self.surfaces.append(surface)
            id = len(self.surfaces) - 1

            #draw vertical lines
            for x in range(width):
                hue = float(x)/width * 360
                col = hsv2rgb((hue,1,1))
                self.drawLine(col, (start+x,y), (start+x,y+height), 1, id)

        return id #return id of new surface

    def makeGrey(self, start, finish, y, height):
        """lalalala nothing to see here"""
        id = -1
        width = finish-start
        if width > 0:
            #create new surface
            surface = pygame.Surface((width, height))
            surface.fill((100,100,100))
            self.surfaces.append(surface)
            id = len(self.surfaces) - 1

        return id

    def drawTiled(self, surfaceid, topleft, bottomright, offset):
        """Draw the surface over the whole area described by topleft and bottomright."""
        if surfaceid >= len(self.surfaces):
            return None
        s = self.surfaces[surfaceid]

        tilewidth = s.get_width()
        tileheight = s.get_height()

        #where tiles start
        x = topleft[0] - tilewidth + int(offset[0] % tilewidth) + 1
        y = topleft[1] - tileheight + int(offset[1] % tileheight) + 1
        top = y

        #draw tiles
        while x <= bottomright[0]:
            while y <= bottomright[1]:
                if x < topleft[0]:
                    overlapx = topleft[0]-x

                    if y < topleft[1]: #goes over the topleft corner
                        overlapy = topleft[1]-y

                        rect = pygame.Rect((overlapx,overlapy),(tilewidth-overlapx,tileheight-overlapy)) #part of the tile we want
                        self.screen.blit(s,topleft,rect)

                    else: #goes over the left side
                        rect = pygame.Rect((overlapx,0),(tilewidth-overlapx,tileheight)) #part of the tile we want
                        self.screen.blit(s,(topleft[0],y),rect)

                elif y < topleft[1]: #goes over the right side
                    overlapy = topleft[1]-y

                    rect = pygame.Rect((0,overlapy),(tilewidth,tileheight-overlapy)) #part of the tile we want
                    self.screen.blit(s,(x,topleft[1]),rect)
                else:
                    self.screen.blit(s,(x,y))
                y += tileheight
            x += tilewidth
            y = top
                
class fontClass:
    """Write text to the screen"""
    def __init__(self):
        pygame.font.init()
        self.rendered = {} #store strings which have aleady been rendered. keys are tuples of text, fontid, color
        self.fonts = {} #dictionary of fonts indexed by name
#        self.chars = [] #prerendered numerals
        self.screen = pygame.display.get_surface()

    def makeFont(self, fontName, fontSize):
        filename = os.path.join("fonts",fontName)
        filename = data.filepath(filename) #add the correct path to the data dir to the start
        f = pygame.font.Font(filename,fontSize)
        self.fonts[(fontName,fontSize)] = f
 #       return len(self.fonts) - 1

 #   def makeNumbers(self,fontid,col):
#        """Renders all the digits"""
#        for c in string.digits:
#            s = f.render(c,True,col)
#            self.chars.append(s)

    def writeText(self, text,fontName,fontSize,col,pos,centered=True,save=True):
        """Write text"""
        if (fontName,fontSize) not in self.fonts:
            self.makeFont(fontName,fontSize)
        
        if (text,fontName,fontSize,col) in self.rendered:
            #already rendered
            s = self.rendered[(text,fontName,fontSize,col)]
        else:
            #render text
            s = self.fonts[(fontName,fontSize)].render(text, True, col)
            if save:
                self.rendered[(text,fontName,fontSize,col)] = s
            
        size = s.get_size()
        if centered:
            dest = (pos[0]-size[0]/2, pos[1]-size[1]/2)
        else:
            dest = pos
        self.screen.blit(s,dest)
        
  #  def writeNum(self,i,pos,centred=True):
#        """Write a number using prerendered surfaces"""
#        if not self.chars:
#            return None
#        text = str(i)
        #fuck it ill figure out how to do this later

class animation(pygame.sprite.Sprite):
    def __init__(self, initial_position=(0,0), frames=[]):

        pygame.sprite.Sprite.__init__(self)
        self.frames = frames
        self.image = self.frames[0]
        self.currentFrame = 0
        self.next_update_time = 0
        self.screen = pygame.display.get_surface()

        #Make our top-left corner the passed-in location.
        self.rect = self.frames[0].get_rect()
        self.rect.topleft = initial_position

    def draw(self):
        self.screen.blit(self.image, self.rect)

    def update(self, current_time):
        # 25fps
        if self.next_update_time < current_time:
            self.currentFrame = (self.currentFrame + 1) % len(self.frames)
            self.image = self.frames[self.currentFrame]
            self.next_update_time = current_time + 40


class ducky(animation):
    """An animation of a duck which rolls from left to right, stops, then rolls back again.
    if velocity[1] is negative then it will roll the opposite way"""
    frames = []
    explosion = None

    def __init__(self, initial_position, rotations, velocity):
        if not ducky.frames: #this is the first duck! we need to load the images
            frames = ["0001.bmp","0002.bmp","0003.bmp","0004.bmp","0005.bmp","0006.bmp","0007.bmp","0008.bmp","0009.bmp","0010.bmp","0011.bmp","0012.bmp","0013.bmp","0014.bmp","0015.bmp"]
            frames = [data.filepath(i) for i in frames]
            for i in frames:
                surface = pygame.image.load(i)
                surface.set_colorkey((14,56,102))
                ducky.frames.append(surface)
        if not ducky.explosion:
            filename = data.filepath("heart.bmp")
            ducky.explosion = pygame.image.load(filename)
            ducky.explosion = ducky.explosion.convert()
            ducky.explosion.set_colorkey((14,56,102))

        animation.__init__(self,initial_position,ducky.frames)
        self.rotations = rotations #how many times it rolls before coming to a stop
        self.completed = 0 #completed rotations
        if velocity[0] > 0:
            self.startdirection = 1
            self.velocity = velocity
        else:
            self.startdirection = -1
            self.velocity = (-velocity[0],velocity[1])
        self.direction = self.startdirection #1 = moving right, -1 is moving left
        self.next_move_time = 0
        self.dying = False
        self.dead = False

    def finished(self):
        if self.direction == -1*self.startdirection and self.completed == self.rotations:
            return True
        elif self.dead:
            return True
        else:
            return False

    def die(self,now):
        self.dying = True
        self.dieTime = now + duckDeathTime
        self.image = ducky.explosion.copy()
        #self.image.set_colorkey((14,56,102))

    def collidePoint(self,x):
        """Find the lowest point on a vertical line which collides with the ducky, or returns none if the line doesnt cross it"""
        
        #check the line crosses its rect
        if x<=self.rect.left or x>=self.rect.right:
            return None
        
        transparent = self.image.get_at((0, 0))

        #get the lowest one which isnt transparent
        result = None
        for i in range(self.rect.height-1,0,-1):
            col = self.image.get_at((x-self.rect.left, i))
            if col != transparent:
                result = i + self.rect.top
                break
        return result
        
    def update(self, current_time):
        # 25fps
        if self.next_update_time < current_time:
            y = self.rect.topleft[1] + self.velocity[1] * 40
            x = self.rect.topleft[0]

            if self.dying:
                if current_time > self.dieTime:
                    self.dying = False
                    self.dead = True # =( poor ducky
                else:
                    alpha = self.image.get_alpha()
                    if alpha > 0:
                        alpha -= fadeAmount
                        if alpha < 0: alpha = 0
                        self.image.set_alpha(alpha)
                    pass

            elif self.next_move_time < current_time and not self.finished():
                self.currentFrame += self.direction #play the animation backwards when its going left

                if self.currentFrame >= len(self.frames) or self.currentFrame <0: #finished a rotation
                    self.completed += 1
                    if self.completed == self.rotations:
                        if not self.finished():
                            self.direction *= -1 #stop, and then go back the other way
                            self.completed = 0
                            self.next_move_time = current_time + duckDelay
                    else:
                        if self.direction == 1: #they are going right
                            self.currentFrame = 0 #play another loop of the animation
                        else:
                            self.currentFrame = len(self.frames) - 1
                        self.image = self.frames[self.currentFrame]
                        x += self.direction * self.velocity[0] * 40

                else:
                    #change the image to the next frame and move it a bit more
                    self.image = self.frames[self.currentFrame]
                    x += self.velocity[0] * self.direction * 40        

            self.rect.topleft = (x,y)
            self.next_update_time = current_time + 40
